@@ -1,4 +1,4 @@
-use crate::constants::MAX_PRECISION_U32;
+use crate::{constants::MAX_PRECISION_U32, Decimal};
 use alloc::string::String;
 use core::fmt;
 
@@ -8,7 +8,9 @@ pub enum Error {
     ErrorString(String),
     ExceedsMaximumPossibleValue,
     LessThanMinimumPossibleValue,
+    Underflow,
     ScaleExceedsMaximumPrecision(u32),
+    ConversionTo(String),
 }
 
 impl<S> From<S> for Error
@@ -21,6 +23,11 @@ where
     }
 }
 
+#[cold]
+pub(crate) fn tail_error(from: &'static str) -> Result<Decimal, Error> {
+    Err(from.into())
+}
+
 #[cfg(feature = "std")]
 impl std::error::Error for Error {}
 
@@ -34,6 +41,9 @@ impl fmt::Display for Error {
             Self::LessThanMinimumPossibleValue => {
                 write!(f, "Number less than minimum value that can be represented.")
             }
+            Self::Underflow => {
+                write!(f, "Number has a high precision that can not be represented.")
+            }
             Self::ScaleExceedsMaximumPrecision(ref scale) => {
                 write!(
                     f,
@@ -41,6 +51,9 @@ impl fmt::Display for Error {
                     scale, MAX_PRECISION_U32
                 )
             }
+            Self::ConversionTo(ref type_name) => {
+                write!(f, "Error while converting to {}", type_name)
+            }
         }
     }
 }
