@@ -1,9 +1,9 @@
+use crate::constants::MAX_PRECISION_U32;
 use crate::{
-    decimal::{div_by_u32, is_all_zero, mul_by_u32, MAX_PRECISION},
+    ops::array::{div_by_u32, is_all_zero, mul_by_u32},
     Decimal,
 };
-use core::{convert::TryInto, fmt, result::*};
-use num_traits::Zero;
+use core::{convert::TryInto, fmt};
 use std::error;
 
 #[derive(Debug, Clone)]
@@ -38,13 +38,13 @@ impl Decimal {
             digits,
             weight,
         }: PostgresDecimal<D>,
-    ) -> Result<Self, InvalidDecimal> {
+    ) -> Self {
         let mut digits = digits.into_iter().collect::<Vec<_>>();
 
         let fractionals_part_count = digits.len() as i32 + (-weight as i32) - 1;
         let integers_part_count = weight as i32 + 1;
 
-        let mut result = Decimal::zero();
+        let mut result = Decimal::ZERO;
         // adding integer part
         if integers_part_count > 0 {
             let (start_integers, last) = if integers_part_count > digits.len() as i32 {
@@ -61,17 +61,16 @@ impl Decimal {
         }
         // adding fractional part
         if fractionals_part_count > 0 {
-            let dec: Vec<_> = digits.into_iter().collect();
             let start_fractionals = if weight < 0 { (-weight as u32) - 1 } else { 0 };
-            for (i, digit) in dec.into_iter().enumerate() {
+            for (i, digit) in digits.into_iter().enumerate() {
                 let fract_pow = 4 * (i as u32 + 1 + start_fractionals);
-                if fract_pow <= MAX_PRECISION {
+                if fract_pow <= MAX_PRECISION_U32 {
                     result += Decimal::new(digit as i64, 0) / Decimal::from_i128_with_scale(10i128.pow(fract_pow), 0);
-                } else if fract_pow == MAX_PRECISION + 4 {
+                } else if fract_pow == MAX_PRECISION_U32 + 4 {
                     // rounding last digit
                     if digit >= 5000 {
                         result +=
-                            Decimal::new(1 as i64, 0) / Decimal::from_i128_with_scale(10i128.pow(MAX_PRECISION), 0);
+                            Decimal::new(1_i64, 0) / Decimal::from_i128_with_scale(10i128.pow(MAX_PRECISION_U32), 0);
                     }
                 }
             }
@@ -80,8 +79,7 @@ impl Decimal {
         result.set_sign_negative(neg);
         // Rescale to the postgres value, automatically rounding as needed.
         result.rescale(scale as u32);
-
-        Ok(result)
+        result
     }
 
     fn to_postgres(self) -> PostgresDecimal<Vec<i16>> {
@@ -141,8 +139,8 @@ impl Decimal {
     }
 }
 
-#[cfg(feature = "diesel")]
-mod diesel {
+#[cfg(feature = "db-diesel-postgres")]
+mod diesel_postgres {
     use super::*;
     use ::diesel::{
         deserialize::{self, FromSql},
@@ -177,8 +175,7 @@ mod diesel {
                 weight,
                 scale,
                 digits: digits.iter().copied().map(|v| v.try_into().unwrap()),
-            })
-            .map_err(Box::new)?)
+            }))
         }
     }
 
@@ -191,11 +188,6 @@ mod diesel {
     }
 
     impl<'a> From<&'a Decimal> for PgNumeric {
-        // NOTE(clippy): Clippy suggests to replace the `.take_while(|i| i.is_zero())`
-        // with `.take_while(Zero::is_zero)`, but that's a false positive.
-        // The closure gets an `&&i16` due to autoderef `<i16 as Zero>::is_zero(&self) -> bool`
-        // is called. There is no impl for `&i16` that would work with this closure.
-        #[allow(clippy::assign_op_pattern, clippy::redundant_closure)]
         fn from(decimal: &'a Decimal) -> Self {
             let PostgresDecimal {
                 neg,
@@ -204,8 +196,6 @@ mod diesel {
                 digits,
             } = decimal.to_postgres();
 
-            let digits = digits.into_iter().map(|v| v.try_into().unwrap()).collect();
-
             if neg {
                 PgNumeric::Negative { digits, scale, weight }
             } else {
@@ -215,8 +205,8 @@ mod diesel {
     }
 
     impl From<Decimal> for PgNumeric {
-        fn from(bigdecimal: Decimal) -> Self {
-            (&bigdecimal).into()
+        fn from(decimal: Decimal) -> Self {
+            (&decimal).into()
         }
     }
 
@@ -234,7 +224,7 @@ mod diesel {
     }
 
     #[cfg(test)]
-    mod pg_tests {
+    mod tests {
         use super::*;
         use core::str::FromStr;
 
@@ -359,7 +349,6 @@ mod diesel {
         }
 
         #[test]
-        #[cfg(feature = "unstable")]
         fn decimal_to_pg_numeric_retains_sign() {
             let decimal = Decimal::from_str("123.456").unwrap();
             let expected = PgNumeric::Positive {
@@ -470,10 +459,10 @@ mod diesel {
     }
 }
 
-#[cfg(feature = "postgres")]
-mod postgres {
+#[cfg(any(feature = "db-postgres", feature = "db-tokio-postgres"))]
+mod _postgres {
     use super::*;
-    use ::postgres::types::*;
+    use ::postgres::types::{to_sql_checked, FromSql, IsNull, ToSql, Type};
     use byteorder::{BigEndian, ReadBytesExt};
     use bytes::{BufMut, BytesMut};
     use std::io::Cursor;
@@ -552,15 +541,11 @@ mod postgres {
                 weight,
                 scale,
                 digits: groups.into_iter(),
-            })
-            .map_err(Box::new)?)
+            }))
         }
 
         fn accepts(ty: &Type) -> bool {
-            match ty {
-                &Type::NUMERIC => true,
-                _ => false,
-            }
+            matches!(*ty, Type::NUMERIC)
         }
     }
 
@@ -599,10 +584,7 @@ mod postgres {
         }
 
         fn accepts(ty: &Type) -> bool {
-            match ty {
-                &Type::NUMERIC => true,
-                _ => false,
-            }
+            matches!(*ty, Type::NUMERIC)
         }
 
         to_sql_checked!();
